@@ -3,6 +3,7 @@ import os
 
 from httplib import socket 
 from threading import Thread
+from collections import deque
 
 from PyQt4.QtCore import QObject, QTimer
 from data import DataParser, Collection, Resource
@@ -21,55 +22,99 @@ class ConnectionManager:
         self.tree_controller = ui_handler.tree_controller
         self.logger = logger
         self.client = None
+        self.transfer_manager = None
 
         # Init poll timer
         self.poll_timer = QTimer()
         self.poll_timer.timeout.connect(self.thread_poller)
 
-        # Init running threads to empty
+        # Init thread params
         self.running_threads = []
         self.queued_threads = []
+        self.active_data_worker = None
+        self.data_workers = deque()
         
         # DBus manager
         self.dbus_monitor = DBusMonitor(self, logger, maemo_env)
         
         # Start the thread poller
-        self.poll_timer.start(100)
+        self.poll_timer.start(200)
 
-    def thread_poller(self):
-        self.dbus_monitor.iteration()
-        for thread in self.running_threads:
-            thread.join(0.01)
-            if not thread.isAlive():
-                if thread.error != None:
-                    # If tree exists, lets store the request and do it again
-                    # when a connection is available
-                    if self.tree_controller.root_folder != None:
-                        self.queued_threads.append(thread)
-                    # Request a connection only once
-                    if len(self.queued_threads) == 1:
-                        self.request_connection()
-                if thread.response != None:
-                    if thread.callback_parameters != None:
-                        thread.callback(thread.response, thread.callback_parameters)
-                    else:
-                        thread.callback(thread.response)
-                self.running_threads.remove(thread)
+    def set_transfer_manager(self, transfer_manager):
+        self.transfer_manager = transfer_manager
     
+    def check_data_workers(self):
+        # Start next data worker if available
+        if self.active_data_worker == None:
+            try:
+                worker = self.data_workers.popleft()
+                worker.start()
+                self.active_data_worker = worker
+            except IndexError:
+                self.active_transfer = None
+    
+    def thread_poller(self):
+        # Give gmainloop a iteration
+        self.dbus_monitor.iteration()
+        
+        # Check for completed network threads
+        if len(self.running_threads) > 0:
+            removable_network_threads = []
+            for thread in self.running_threads:
+                thread.join(0.01)
+                if not thread.isAlive():
+                    if thread.error != None:
+                        # If tree exists, lets store the request and do it again
+                        # when a connection is available
+                        if self.tree_controller.root_folder != None:
+                            self.queued_threads.append(thread)
+                        # Request a connection only once
+                        if len(self.queued_threads) == 1:
+                            self.request_connection()
+                    if thread.response != None:
+                        if thread.callback_parameters != None:
+                            thread.callback(thread.response, thread.callback_parameters)
+                        else:
+                            thread.callback(thread.response)
+                    removable_network_threads.append(thread)  
+            for removable in removable_network_threads:
+                self.running_threads.remove(removable)
+                del removable
+        
+        # Check for active data worker
+        if self.active_data_worker != None:
+            self.active_data_worker.join(0.01)
+            if not self.active_data_worker.isAlive():
+                if self.active_data_worker.error != None:
+                    self.logger.error(self.active_data_worker.error)
+                self.active_data_worker = None
+                self.check_data_workers()
+                    
     def request_connection(self):
         self.dbus_monitor.request_connection()
         
     def connection_available(self):
         return self.dbus_monitor.device_has_networking
-        
+    
+    def connection_is_wlan(self):
+        if not self.connection_available():
+            return False
+        bearer = self.dbus_monitor.bearer
+        if bearer == None:
+            return False
+        if bearer.startswith("WLAN"):
+            return True
+        else:
+            return False
+            
     def set_connected(self, connected):
         if connected:
             self.ui_handler.hide_loading_ui()
             if self.controller.login_done == False:
-                self.logger.network("Network connection established, starting authentication")
                 self.controller.start_trusted_auth()
                 trusted_login_ui = self.ui_handler.trusted_login_ui
                 if trusted_login_ui.line_edit_email.text().isEmpty() == False and trusted_login_ui.line_edit_password.text().isEmpty() == False:
+                    self.logger.network("Network connection established, starting authentication")
                     self.ui_handler.try_trusted_login()
             elif self.controller.connected == True and self.client != None:
                 if len(self.queued_threads) > 0:
@@ -87,21 +132,22 @@ class ConnectionManager:
                     self.get_account_data()
                     self.get_metadata("/", "dropbox")
         else:
-            self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+            self.ui_handler.show_loading_ui("Waiting for a connection...", True)
     
     def set_client(self, client):
         # Set client for usage, we are not connected
         self.client = client
+        self.transfer_manager.set_client(client)
         # Get account information
         if self.connection_available():
             if self.get_account_data():
                 # Start by fetching sandbox root contents
                 self.get_metadata("/", "dropbox")
             else:
-                self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+                self.ui_handler.show_loading_ui("Waiting for a connection...", True)
                 self.request_connection()
         else:
-            self.ui_handler.show_loading_ui("Waiting for a connection...", False)
+            self.ui_handler.show_loading_ui("Waiting for a connection...", True)
             self.request_connection()
 
     def get_account_data(self):
@@ -124,8 +170,8 @@ class ConnectionManager:
     def hide_loading_ui(self):
         self.ui_handler.hide_loading_ui()
 
-    def show_information(self, message, succesfull):
-        self.ui_handler.show_information_ui(message, succesfull)
+    def show_information(self, message, succesfull, timeout = 4000):
+        self.ui_handler.show_information_ui(message, succesfull, timeout)
 
     def get_automated_metadata(self, folder_path, root):
         folder = self.tree_controller.get_folder_for_path(folder_path)
@@ -218,43 +264,36 @@ class ConnectionManager:
             self.show_information("Thumbnail fetch failed, internal error", False)
 
     ### GET FILE HANDLERS
-    def get_file(self, path, root, store_path):
+    def get_file(self, path, root, store_path, size, mime_type, sync_download = False):
+        if not self.check_client:
+            return
         file_name = path.split("/")[-1]
-        self.show_loading_ui("Downloading file " + file_name)
-        
-        # Make thread
-        worker = NetworkWorker()
-        worker.set_callable(self.client.get_file, root, path)
-        worker.set_callback(self.get_file_callback, store_path, file_name)
-        worker.start()
-
-        # Add thread to watch list
-        self.running_threads.append(worker)
+        dropbox_path = path[0:path.rfind("/")]
+        if sync_download == False:
+            self.show_information("Download queued\n" + file_name, None, 4000)
+        self.transfer_manager.handle_download(path, root, file_name, dropbox_path, store_path, size, mime_type, sync_download)
         
     def get_file_callback(self, resp, params):
         if resp != None and params[0] != None and params[1] != None:
             store_path = params[0]
             file_name = params[1]
-            self.hide_loading_ui()
             if resp.status == 200:
-                try:
-                    f = open(store_path, "wb")
-                    f.write(resp.read())
-                    f.close()
-                    self.show_information("Download of " + file_name + " completed", True)
-                except IOError:
-                    self.show_information("Could not open " + store_path + " for store operation", False)
+                store_worker = DataWorker("store")
+                store_worker.setup_store(store_path, resp)
+                self.data_workers.append(store_worker)
+                self.check_data_workers()
             else:
                 self.logger.network_error(str(resp.status) + " - Could not download file to", store_path)
                 self.logger.network_error(">> Reason:", resp.read())
-                self.show_information("Download failed, network error", False)
         else:
-            self.hide_loading_ui()
-            self.show_information("Download failed, internal error", False)
+            self.logger.error("Download failed, internal error")
 
     ### UPLOAD FILE HANDLERS
     def upload_file(self, path, root, local_file_path):
-        # Get info for ui and show loading ui
+        if not self.check_client:
+            return
+ 
+        # Get info for ui
         if root == "sandbox":
             root_name = "DropN900"
         elif root == "dropbox":
@@ -263,42 +302,31 @@ class ConnectionManager:
             self.logger.error("Upload file path/root parse error! Cannot continue.")
             return
         file_name = local_file_path.split("/")[-1]
-        self.show_loading_ui("Uploading file " + file_name + "\nto " + root_name + path)
+        folder_path = local_file_path[0:local_file_path.rfind("/")]
 
         # Open file for lib
         try:
             file_obj = open(local_file_path, "rb")
         except IOError:
-            self.hide_loading_ui()
             self.logger.error("Could not open " + local_file_path + " file. Aborting upload.")
             self.show_information("Could not open " + local_file_path + " file for uploading", False)
             return
         
-        # Make thread
-        worker = NetworkWorker()
-        worker.set_callable(self.client.put_file, root, path, file_obj)
-        worker.set_callback(self.upload_file_callback, root, path, file_name)
-        worker.start()
-
-        # Add thread to watch list
-        self.running_threads.append(worker)
+        self.show_information("Upload queued\n" + file_name, None, 4000)
+        self.transfer_manager.handle_upload(path, root, file_obj, file_name, folder_path, root_name + path, local_file_path)
 
     def upload_file_callback(self, resp, params):
         if resp != None and params[0] != None and params[1] != None and params[2] != None:
             root = params[0]
             store_path = params[1]
             file_name = params[2]
-            self.hide_loading_ui()
             if resp.status == 200:
                 self.get_automated_metadata(store_path, root)
-                self.show_information("Upload of " + file_name + " completed", True)
             else:
                 self.logger.network_error(str(resp.status) + " - Upload of " + params[0] + " to " + store_path + "failed.")
                 self.logger.network_error(">> Reason:", resp.body)
-                self.show_information("Upload of " + file_name + " failed, network error", False)
         else:
-            self.hide_loading_ui()
-            self.show_information("Upload failed, internal error", False)
+            self.logger.error("Upload failed, internal error")
 
     ### RENAME HANDLERS
     def rename(self, root, from_path, to_path, data):
@@ -451,3 +479,27 @@ class NetworkWorker(Thread):
             self.response = None
             self.error = err
 
+""" DataWorker is a simple thread subclass that does file writing, this way the main thread can run at peace """
+
+class DataWorker(Thread):
+
+    def __init__(self, action):
+        Thread.__init__(self)
+        self.action = action
+        self.file_path = None
+        self.data = None
+        self.error = None
+        
+    def setup_store(self, file_path, data):
+        self.file_path = file_path
+        self.data = data
+        
+    def run(self):
+        if self.action == "store":
+            try:
+                f = open(self.file_path, "wb")
+                f.write(self.data.read())
+                f.close()
+            except IOError:
+                self.error = "Could not open " + self.file_path + " for file I/O"
+
